<?php
/**
 * Doodle Plugin: helps to schedule meetings
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Esther Brunner <wikidesign@gmail.com>  
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_doodle extends DokuWiki_Syntax_Plugin {
  
  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'wikidesign@gmail.com',
      'date'   => '2006-08-19',
      'name'   => 'Doodle Plugin',
      'desc'   => 'helps to schedule meetings',
      'url'    => 'http://wiki.splitbrain.org/plugin:doodle',
    );
  }

  function getType(){ return 'substition';}
  function getPType(){ return 'block';}
  function getSort(){ return 168; }
  
  /**
   * Connect pattern to lexer
   */
  function connectTo($mode){
    $this->Lexer->addSpecialPattern('<doodle.*?>.+?</doodle>', $mode, 'plugin_doodle');
  }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    $match = substr($match, 8, -9);  // strip markup
    list($title, $options) = preg_split('/>/u', $match, 2);
    if (!$options){
      $options = $title;
      $title   = NULL;
    }
    $options = explode('^', $match);
    
    $c = count($options);
    for ($i = 0; $i < $c; $i++){
      $options[$i] = trim($options[$i]);
    }
    
    return array(trim($title), $options);
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data) {
  
    if ($mode == 'xhtml'){
      global $lang;
      
      $options = $data[1];
      $c = count($options)-1;
      $title   = $renderer->_xmlEntities($data[0]);
      $dID     = cleanID($title);
            
      // prevent caching to ensure the poll results are fresh
      $renderer->info['cache'] = false;
            
      // get doodle file contents
      $dfile   = metaFN(md5($dID), '.doodle');
      $doodle  = unserialize(@file_get_contents($dfile));
      
      if ($c == 0){
        // no options given: reset the doodle
        $doodle = NULL;
      }
      
      // output the doodle
      $renderer->table_open();
      if ($title){
        $renderer->tablerow_open();
        $renderer->tableheader_open($c);
        $renderer->doc .= $title;
        $renderer->tableheader_close();
        $renderer->tablerow_close();
      }
      $renderer->tablerow_open();
      $renderer->tableheader_open();
      $renderer->doc .= $lang['fullname'];
      $renderer->tableheader_close();
      for ($i = 1; $i < $c; $i++){
        $renderer->tableheader_open();
        $renderer->doc .= $renderer->_xmlEntities($options[$i]);
        $renderer->tableheader_close();
      }
      $renderer->tablerow_close();
              
      if ($submit = $_REQUEST[$dID.'-submit']){
        
        // user has just voted -> update results
        $user = trim($_REQUEST['fullname']);
        if (!empty($user)){
          for ($i = 1; $i < $c; $i++){
            $opt = $renderer->_xmlEntities($options[$i]);
            if ($_REQUEST[$dID.'-option'.$i]){
              $doodle[$user][$opt] = true;
            } else {
              $doodle[$user][$opt] = false;
            }
          }
        }
        $fh = fopen($dfile, 'w');
        fwrite($fh, serialize($doodle));
        fclose($fh);
      }
      
      // display results
      if (is_array($doodle)) $renderer->doc .= $this->_doodleResults($doodle, $options);
      
      // display entry form
      $renderer->doc .= $this->_doodleForm($c, $dID, $doodle, $options);

      $renderer->table_close();
          
      return true;
    }
    return false;
  }
  
  function _doodleResults($doodle, $options){
    $cuser = count($doodle);
    if ($cuser < 1) return '';
    $copt  = count($options)-1;
    $users = array_keys($doodle);
    $ret   = '';
    $count = array();
    
    // table okay / not okay
    for ($i = 0; $i < $cuser; $i++){
      $user = $users[$i];
      $ret .= '<tr><td class="rightalign">'.$user.'</td>';
      for ($j = 1; $j < $copt; $j++){
        $option = $options[$j];
        if ($doodle[$user][$option]){
          $class = 'okay';
          $title = '<img src="'.DOKU_BASE.'lib/images/success.png" alt="Okay" '.
            'width="16" height="16" />';
          $count[$option] += 1;
        } elseif (!isset($doodle[$user][$option])){
          $class = 'centeralign';
          $title = '&nbsp;';
        } else {
          $class = 'notokay';
          $title = '&nbsp;';
        }
        $ret .= '<td class="'.$class.'">'.$title.'</td>';
      }
      $ret .= '</tr>';
    }
    
    // total count
    $ret .= '<tr><td>&nbsp;</td>';
    for ($j = 1; $j < $copt; $j++){
      $option = $options[$j];
      $ret .= '<td class="centeralign">'.$count[$option].'</td>';
    }
    $ret .= '</tr>';
    
    return $ret;
  }
  
  function _doodleForm($n, $dID, $doodle, $options){
    global $lang;
    global $ID;
    global $INFO;

    $user  = ($_SERVER['REMOTE_USER'] ? $INFO['userinfo']['name'] : '');
    $count = array();

    $ret = '<form id="doodle__form" method="post" action="'.script().
      '" accept-charset="'.$lang['encoding'].'"><tr>'.
      '<input type="hidden" name="do" value="show" />'.
      '<input type="hidden" name="id" value="'.$ID.'" />'.
      '<td class="rightalign"><input type="text" name="fullname" '.
      'value="'.$user.'" /></td>';
    $i = 1;
    while ($i < $n){
      if (is_array($doodle)){
        $option = $options[$i];
        if ($doodle[$user][$option]){
	        $checked = 'checked="checked" ';
        } else {
          $checked = ' ';
        }
      } else {
        $checked = ' ';
      }
      $ret.= '<td class="centeralign"><input type="checkbox" '.
        'name="'.$dID.'-option'.$i.'" value="1" '.$checked.'/></td>';
      $i++;
    }
    $ret .= '</tr><tr><td class="centeralign" colspan="'.($n).'">'.
      '<input class="button" type="submit" name="'.$dID.'-submit" '.
      'value="'.$this->getLang('btn_submit').'" />'.
      '</td></tr></form>';

    return $ret;
  }
}
 
//Setup VIM: ex: et ts=4 enc=utf-8 :
