<?php
/**
 * Folded text Plugin: enables folded text font size with syntax ++ text ++
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Fabian van-de-l_Isle <webmaster [at] lajzar [dot] co [dot] uk>
 * @author     Christopher Smith <chris@jalakai.co.uk>
 * @author     Esther Brunner <esther@kaffeehaus.ch>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
// maintain a global count of the number of folded elements in the page, 
// this allows each to be uniquely identified
global $plugin_folded_count;
if (!isset($plugin_folded_count)) $plugin_folded_count = 0;
 
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_folded_div extends DokuWiki_Syntax_Plugin {
 
   /**
    * return some info
    */
    function getInfo(){
        return array(
            'author' => 'Fabian van-de-l_Isle',
            'email'  => 'webmaster@lajzar.co.uk',
            'date'   => '2005-09-18',
            'name'   => 'Folded Plugin - Block',
            'desc'   => 'Enable blocks of folded text. 
                         Syntax: ++++title|folded content++++',
            'url'    => 'http://wiki.splitbrain.org/plugin:folded',
        );
    }
 
    function getType(){ return 'container'; }
    function getAllowedTypes() { return array('container','substition','protected','disabled','formatting'); }
    function getSort(){ return 404; }
    function connectTo($mode) { $this->Lexer->addEntryPattern('\+\+\+\+.*?\|(?=.*\+\+\+\+)',$mode,'plugin_folded_div'); }
    function postConnect() { $this->Lexer->addExitPattern('\+\+\+\+','plugin_folded_div'); }
 
   /**
    * Handle the match
    */
    function handle($match, $state, $pos, &$handler){
        if ($state == DOKU_LEXER_ENTER){
            $match = trim(substr($match,4,-1)); // strip markup
        }
        return array($state, $match);
    }
 
   /**
    * Create output
    */
    function render($mode, &$renderer, $data) {
        global $plugin_folded_count;
 
        if($mode == 'xhtml') {
            switch ($data[0]){
              case DOKU_LEXER_ENTER:
                $plugin_folded_count++;
                $renderer->doc .= '<a class="folder"'.
                    ' href="javascript:fold(this, '.$plugin_folded_count.');"'.
                    ' title="'.$this->getLang('reveal').'">';
                if ($data[1])
                    $renderer->doc .= ' '.$renderer->_xmlEntities($data[1]);
                $renderer->doc .= '<img id="folder_'.$plugin_folded_count.'"'.
                    ' src="'.DOKU_BASE.'lib/plugins/folded/closed.gif"'.
                    ' alt="'.$this->getLang('reveallong').'" /></a></p>'.
                    '<div class="folded" id="folded_'.$plugin_folded_count.'"'.
                    ' style="display:none;"><p>';
                break;
                
              case DOKU_LEXER_UNMATCHED:
                $renderer->doc .= $renderer->_xmlEntities($data[1]);
                break;
                
              case DOKU_LEXER_EXIT:
                $renderer->doc .= '</p></div><p>';
                break;
            }
            return true;
        }
        return false;
    }
    
}