<?php
/**
 * Setting up the sympa-soap authentication admin plugin
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     David Pepin <sympa-authors@cru.fr>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'admin.php');




/**
 * All DokuWiki plugins to extend the admin function
 * need to inherit from this class
 */
class admin_plugin_sympaauth extends DokuWiki_Admin_Plugin {


  /**
   * Return some info about the plugin
   *
   **/
  function getInfo(){

      return array(
		   'author' => 'David Pepin',
		   'email'  => 'sympa-authors@cru.fr',
		   'date'   => '2006/07/03',
		   'name'   => 'sympaauth',
		   'desc'   => 'Plugin to grant authentication and access rights from a Soap Sympa server',
		   'url'    => 'http://www.sympa.org',
		   );
  }

  /**
   * handle user request
   */
  function handle() {
    if ($_REQUEST['action_'] == 'install') {
      $this->install();
    }
    elseif ($_REQUEST['action_'] == 'uninstall') {
      $this->uninstall();
    }      
  }

  /**
   * output appropriate html
   */
  function html() {
    global $ID;
    // Display the text about this plugin from "sympaauth/lang/$lang/intro.txt"
    print $this->locale_xhtml('intro');

    // If not installed, it displays the install menu
    // if installed, it displays the uninstall menu
    ptln('<form action_="'.wl($ID).'" method="post">');
    if($this->is_installed()) {
      /*
      $authmodes = $this->get_authmodes();
      ptln('<p> ' . $this->getLang('newAuthMode'));
      ptln('       <select name="authmode">');
      foreach($authmodes as $authmode) {
	ptln('          <option value="'.$authmode .'"> '. $authmode .' </option>');
      }
      ptln('       </select>');
      ptln('</p>');
      ptln('<p> '.$this->getLang('uninstall') .' </p>');
 
      ptln('<p> <input type="hidden" name="action_" value="uninstall" />');
      ptln('    <input type="submit" name="submit" class="button" value="'.$this->getLang('btn_uninstall').'" /> </p>');
      */
      if($this->is_used()) {
	ptln('<p> ' .$this->getLang('is_used'). '</p>');
      }
      else {
	ptln('<p> '.$this->getLang('uninstall') .' </p>');
	ptln('<p> <input type="hidden" name="action_" value="uninstall" />');
	ptln('    <input type="submit" name="submit" class="button" value="'.$this->getLang('btn_uninstall').'" /> </p>');
      }
    }
    else {
      ptln('<p> '.$this->getLang('install') .' </p>');
      ptln('<p> <input type="hidden" name="action_" value="install" />');
      ptln('    <input type="submit" name="submit" class="button" value="'.$this->getLang('btn_install').'" /> </p>');
    }
    ptln('  <p> <input type="hidden" name="do"     value="admin" />');
    ptln('      <input type="hidden" name="page"   value="sympaauth" /> </p>');
    ptln('</form>');

    
  }


  function is_installed() {
    if(!defined('DOKU')) define('DOKU', realpath(dirname(__FILE__) .'/../../../'));
    
    //Check if the file sympa.class.php is in the authenticafion methods directory
    if (!file_exists(DOKU . '/inc/auth/sympa.class.php')) {
      return false;
    }
    
    //Check if the nusoap library file is in the lib directory
    if (!file_exists(DOKU . '/lib/nusoap.php')) {
      return false;
    }
    
    //Check if the sympa soap server is rightly set up in the file local.php
    if (!$local_handle = fopen(DOKU . '/conf/local.php', 'r')) {
      return false;
    } 
    while (!feof($local_handle)) {
      $line = fgets($local_handle, 1000);
      if(preg_match('!\$conf\[\'sympaSoapServer\'\] *= *.*!', $line)) {
	fclose($local_handle);
	return true;
      }
    }
    fclose($local_handle);
    return false;
  }
   
  function is_used() {
    //Check if the authentication mode is the sympa-soap server one
    if (!$local_handle = fopen(DOKU . '/conf/local.php', 'r')) {
      return true;
    } 
    while (!feof($local_handle)) {
      $line = fgets($local_handle, 1000);
      if(preg_match('!\$conf\[\'authtype\'\] *= *.*!', $line)) {
	$value = $line;
      }
      if(preg_match('!\$conf\[\'authtype\'\] *= *[\'"]sympa[\'"]!', $value)) return true;
    }
    fclose($local_handle);
    return false;
  }


  function install() {
    if(!defined('DOKU')) define('DOKU', realpath(dirname(__FILE__) .'/../../../'));
    // Check if the plugin files are already there, and delete it if it's the case
    if (file_exists(DOKU . '/inc/auth/sympa.class.php')) {
      if(!unlink(DOKU . '/inc/auth/sympa.class.php')) return false;
    }
    if (file_exists(DOKU . '/lib/nusoap.php')) {
      if(!unlink(DOKU . '/lib/nusoap.php')) return false;
    }
    // Copy the new ones
    if (!copy(DOKU .'/lib/plugins/sympaauth/files/sympa.class.php', DOKU .'/inc/auth/sympa.class.php')) return false;
    if (!copy(DOKU .'/lib/plugins/sympaauth/files/nusoap.php', DOKU .'/lib/nusoap.php')) return false;
    
    // Backup
    if (file_exists(DOKU . '/conf/local.php')) {
      if (!copy(DOKU . '/conf/local.php', DOKU . '/conf/local.php.bak')) return false;
    }
  
    // Open local.php, copy its content to $lines, look for $conf['sympaSoapServer'] line(s) delete it (them) and create it
    // just before loading the protected settings.
    // Note that $conf['sympaSoapServer'] should be before "@include(DOKU_CONF.'local.protected.php');"
    $i=0;
    //    if (!(feof($local_handle))) { // if the file is not empty, assuming it has a <? 
    if (file_exists(DOKU .'/conf/local.php') && $local_handle = fopen(DOKU . '/conf/local.php', 'r')) { // if the file exists and is readable
      if (!(feof($local_handle))) { // and if the file is not empty, assuming it has a <? 
	do {
	  $lines[$i] = fgets($local_handle, 1000);
	  if(preg_match('!\$conf\[\'sympaSoapServer\'\] *= *.*!', $lines[$i])) unset($lines[$i--]);
	} while (!(preg_match('!local.protected.php!', $lines[$i++]) || feof($local_handle)) );
	$temp = $lines[--$i];
	$lines[$i++] = '$conf[\'sympaSoapServer\'] = \'\';' . "\n";
	$lines[$i] = $temp;
	while (!feof($local_handle)) {
	  $i++;
	  $lines[$i] = fgets($local_handle, 1000);
	  if(preg_match('!\$conf\[\'sympaSoapServer\'\] *= *.*!', $lines[$i])) unset($line[$i--]);
	}    
      }
      else {
	$lines[$i] = '<?'."\n";
	$lines[++$i] = '$conf[\'sympaSoapServer\'] = \'\';' . "\n";
      }
    }
    else { // if the file does not exist
      if (!$local_handle = fopen(DOKU . '/conf/local.php', 'w+')) return false;
      $lines[$i] = '<?'."\n";
      $lines[++$i] = '$conf[\'sympaSoapServer\'] = \'\';' . "\n";
    }
    fclose($local_handle);
    
    // Delete local.php (a backup was created few lines before), then copy $lines to a new local.php. Restore backup if failure.
    if(!unlink(DOKU . '/conf/local.php')) return false;
    if (!$local_handle = fopen(DOKU . '/conf/local.php', 'w')) {
      copy(DOKU . '/conf/local.php.bak', DOKU . '/conf/local.php');
      return false;
    }
    foreach ($lines as $line) {
      if (!empty($line) && fwrite($local_handle, $line) == FALSE) {
	fclose($local_handle);
	unlink(DOKU . '/conf/local.php');
	copy(DOKU . '/conf/local.php.bak', DOKU . '/conf/local.php');
	return false;
      }
    }
    
    // Install localization for the config plugin
    return($this->install_langconf());
    
    return(true);
  }



  function uninstall() {
    if(!defined('DOKU')) define('DOKU', realpath(dirname(__FILE__) .'/../../../'));
    // Check if the plugin files are already there, and delete it if it's the case
    if (file_exists(DOKU . '/inc/auth/sympa.class.php')) {
      if(!unlink(DOKU . '/inc/auth/sympa.class.php')) return false;
    }
    if (file_exists(DOKU . '/lib/nusoap.php')) {
      if(!unlink(DOKU . '/lib/nusoap.php')) return false;
    }
    
    if (file_exists(DOKU . '/conf/local.php')) {
      if (!copy(DOKU . '/conf/local.php', DOKU . '/conf/local.php.bak')) return false;
    }
    
    // Open local.php, copy its content to $lines, look for $conf['sympaSoapServer'] line(s) and delete it (them)
    if (!$local_handle = fopen(DOKU . '/conf/local.php', 'r')) return false;
    $i = -1;
    while (!feof($local_handle)) {
      $i++;
      $lines[$i] = fgets($local_handle, 1000);
      if(preg_match('!\$conf\[\'sympaSoapServer\'\] *= *.*!', $lines[$i])) unset($lines[$i--]);
    }
    fclose($local_handle);
    
    // Delete local.php after having backed it up, then copy $lines to a new local.php. Restore backup if failure.
    if(!unlink(DOKU . '/conf/local.php')) return false;
    if (!$local_handle = fopen(DOKU . '/conf/local.php', 'w+')) {
      copy(DOKU . '/conf/local.php.bak', DOKU . '/conf/local.php');
      return false;
    }
    foreach ($lines as $line) {
      if (!empty($line) && fwrite($local_handle, $line) == FALSE) {
	fclose($local_handle);
	unlink(DOKU . '/conf/local.php');
	copy(DOKU . '/conf/local.php.bak', DOKU . '/conf/local.php');
	return false;
      }
    }
    fclose($local_handle);
    
    // Uninstall localization for the config plugin
    return($this->uninstall_langconf());
  }

  function get_authmodes() {
    if(!defined('DOKU')) define('DOKU', realpath(dirname(__FILE__) .'/../../../'));
    if( ($dir_handle = opendir(DOKU. '/inc/auth')) == FALSE ) return false;
    $i=0;
    while (false !== ($file = readdir($dir_handle))) {
      if ($file != "." && $file != "..") {
	if(preg_match('![a-zA-Z0-9]{1,}\.class\.php!',$file)) {
	  $authmodes[$i] = preg_replace('!([a-zA-Z0-9]{1,})\.class\.php!',"$1",$file);
	  if($authmodes[$i++] == "sympa") {
	    $i--;
	    unset($authmodes[$i]);
	  }
	}
      }
    }
    return $authmodes;
  }


function install_langconf() {
  if(!defined('DOKU_CONFIGLANG')) define('DOKU_CONFIGLANG', realpath(dirname(__FILE__).'/../config/lang/'));
  if(!defined('DOKU_SYMPALANG')) define('DOKU_SYMPALANG', realpath(dirname(__FILE__) .'/lang/'));

  // Read what are the languages available with the config plugin
  if( !$configlang_h = opendir(DOKU_CONFIGLANG) ) return false;
  while ($lang = readdir($configlang_h)) {
    if ($lang != "." && $lang != ".." && is_dir(DOKU_CONFIGLANG.'/'. $lang)) {
      // Then look for every languages in the sympaauth plugin
      if(is_dir(DOKU_SYMPALANG.'/'. $lang)) {
	if(!$sympa_file = fopen(DOKU_SYMPALANG .'/' .$lang .'/config_lang.php','r')) return false;
	$i=0;
	// And copy the $lang lines from each... assuming that noone would place a ; at the beginning of a new line :-)
	while (!feof($sympa_file)) {
	  $lines[$i] = fgets($sympa_file, 1000);
	  if(!preg_match('!\$lang\[\'\w{1,}\'\] *= *.*!', $lines[$i])) unset($lines[$i--]);
	  $i++;
	}
	fclose($sympa_file);
	// To the associated language file of the config plugin
	if(!$config_file= fopen(DOKU_CONFIGLANG .'/'.$lang .'/lang.php','a')) return false;
	if(!fwrite($config_file,'/* Parameters for authenticating from a sympa server */'."\n")) return false;
	foreach ($lines as $line) {
	  if(!empty($line) && !fwrite($config_file,$line ."\n")) return false;
	}
	fclose($config_file);
      }
    }
  }
  closedir($configlang_h);
  return(true);
}



  function uninstall_langconf() {
    if(!defined('DOKU_CONFIGLANG')) define('DOKU_CONFIGLANG', realpath(dirname(__FILE__).'/../config/lang/'));
    if( !$configlang_h = opendir(DOKU_CONFIGLANG) ) return false;
    // Delete every line containing the word "sympa"
    while (false !== ($lang = readdir($configlang_h)) ) {
      if ($lang != "." && $lang != ".." && is_dir(DOKU_CONFIGLANG.'/'. $lang)) {
	if(!$sympa_file = fopen(DOKU_CONFIGLANG .'/' .$lang .'/lang.php','r')) return false;
	$i = 0;
	while (!feof($sympa_file)) {
	  $lines[$i] = fgets($sympa_file, 1000);
	  if(preg_match('!sympa!i', $lines[$i])) unset($lines[$i--]);
	  $i++;
	}
	fclose($sympa_file);
	
	// Delete lang.php after having backed it up, then copy $lines to a new lang.php. Restore backup if failure.
	if(!copy(DOKU_CONFIGLANG .'/' .$lang .'/lang.php', DOKU_CONFIGLANG .'/' .$lang .'/lang.php.bak')) return false;
	if(!unlink(DOKU_CONFIGLANG .'/' .$lang .'/lang.php')) return false;
	if (!$sympa_file = fopen(DOKU_CONFIGLANG .'/' .$lang .'/lang.php', 'w')) {
	  copy(DOKU . '/conf/local.php.bak', DOKU . '/conf/local.php');
	  return false;
	}
	foreach ($lines as $line) {
	  if (!empty($line) && fwrite($sympa_file, $line) == FALSE) {
	    copy(DOKU . '/conf/local.php.bak', DOKU . '/conf/local.php');
	    return false;
	  }
	}
	fclose($sympa_file);
      }
    }
  }

}