<?php

/**
 * Authentication backend using a Sympa Soap server
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     David Pepin <sympa-authors@cru.fr>
 * Version: 0.1
 * last modified: 2006-06-20
 *
 * Work based on :
 *    ssl authentication backend:
 *       @author     Dominique Launay <dominique.launay@cru.fr>
 *    htaccess authentication backend:
 *       @author     Samuele Tognini<samuele@cli.di.unipi.it
 *
 * The nusoap library is required
 *
 **/


define('DOKU_AUTH', dirname(__FILE__));
require_once(DOKU_AUTH.'/basic.class.php');
require_once(DOKU_AUTH.'/../../lib/nusoap.php');



class auth_sympa extends auth_basic {
  var $users = null;
  
  /**
   *
   * Constructor
   *
   * @author  David Pepin <sympa-authors@cru.fr>
   *
   **/
  function auth_sympa() {
    global $conf;
    if (method_exists($this, 'auth_basic'))
      parent::auth_basic();
      
    // check if the server configuration has correctly been done
    if (empty($conf['sympaSoapServer'])) {
      $this->success = false;
      return;
    }
    if (isset($conf['defaultgroup'])) {
      $this->sympaDefaultGroup = $conf['defaultgroup'];
    }
    $this->sympaSoapServer = $conf['sympaSoapServer'];  
    $this->soap_client = new soapclient($this->sympaSoapServer.'/wsdl','wsdl');

    $this->success = true;
    return;
  }



  /**
   * Checks user and password using the Soap frontend to Sympa
   *
   * @author  David Pepin <sympa-authors@cru.fr>
   *    based on nusoap.php which provides functions to access sympa through soap
   *
   **/

  function checkPass($user,$pass){
    $auth_key = $this->soap_client->call('login',array($user,$pass));
    if(!is_array($auth_key)) {
      $_SESSION['sympapass'] = $auth_key;
      return(true);
    }
    else {
      return(false);
    }
  }



  /**
   * Return user info
   *
   * Accessing to sympa is done through the nusoap library
   *
   * The returned field are :
   *
   * name string  full name of the user 
   * mail string  email addres of the user 
   * groups array   list of groups the user is in
   *
   * @author  David Pepin <sympa-authors@cru.fr>
   *
   **/
  function getUserData($user) {
    // Name is mail
    $name = $user;

    // mail address
    $mail = $user;
    
    // groups the person belongs to
    $grps = array();
    $res = $this->soap_client->call('authenticateAndRun',array($user,$_SESSION['sympapass'],'complexWhich'));
    if (isset($res) && gettype($res) == 'array') {
      foreach ($res as $list) {
	(empty($grps[0]) ? $grps[0] = $list['listAddress'] : array_unshift($grps,$list['listAddress']));
      }
    }
    //sympadefaultgroup is put in first position, if set
    if(isset($this->sympaDefaultGroup)) {
      (empty($grps[0]) ? $grps[0] = $this->sympaDefaultGroup : array_unshift($grps,$this->sympaDefaultGroup) );
    }

    // Return the collected pieces of information about the user
    $this->users[$user] = compact('name','mail','grps');
    return (isset($this->users[$user]) ? $this->users[$user] : false);
  }



  /**
   * Logoff user : 
   *
   * everything that was about the session is unset
   *
   * @author David Pepin <sympa-authors@cru.fr>
   *
   **/
  function logOff() {
    unset($this->soap_client);
    unset($_SESSION['sympapass']);
    exit;
  }

  function server() {
    return($this->sympaSoapServer);
  }
}
